# coding: utf-8

import datetime
import sys
import unittest

sys.path.append('../')
import ezcf

from sample_json import *
from sample_yaml import *


class TestProto(unittest.TestCase):

    def test_import_all(self):
        self.assertEqual(hello, "world")
        self.assertEqual(a_list, [1 ,2, 3])
        self.assertEqual(a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        self.assertEqual(Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            Stack,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(User, 'ed')
        self.assertEqual(warning, u'一个 slightly different error message.')


import os
import unittest
import sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from subdir.subdir.test_parent import *

if __name__ == '__main__':
    unittest.main()


"""
Base Finder and Loader Class
"""

import imp
import inspect
import os
import sys


class InvalidJsonError(Exception):
    pass


class InvalidYamlError(Exception):
    pass


class _BaseClass(object):
    pass


class BaseFinder(_BaseClass):

    def __init__(self, *args, **kwargs):
        pass

    def find_module(self, *args, **kwargs):
        raise NotImplementedError()

    @staticmethod
    def get_parent(path, level=1):
        for _ in range(level):
            path = os.path.dirname(path)
        return path

    def get_cfg_filepath(self, fullname):
        caller_frame = sys._getframe(3)
        file = inspect.getfile(caller_frame)  # file that calls 'import '
        top_package_dir = os.path.dirname(file)
        if '__name__' in caller_frame.f_globals:
            if '.' in caller_frame.f_globals['__name__']:  # find top package
                up_level = len(caller_frame.f_globals['__name__'].split('.'))
                top_package_dir = self.get_parent(top_package_dir, up_level-1)

        if '.' in fullname:
            cfg_file = os.path.join(*([top_package_dir] + fullname.split('.')))
        else:
            cfg_file = fullname

        return cfg_file


class BaseLoader(_BaseClass):

    def __init__(self, *args, **kwargs):
        pass

    def load_module(self, fullname):
        if fullname in sys.modules:
            mod = sys.modules[fullname]
        else:
            mod = sys.modules.setdefault(fullname, imp.new_module(fullname))

        mod.__file__ = self.cfg_file
        mod.__name__ = fullname
        mod.__loader__ = self
        mod.__package__ = '.'.join(fullname.split('.')[:-1])

        return mod




from .type_json import JsonFinder
from .type_yaml import YamlFinder


class ConfigFinder(object):

    def __init__(self):
        self.finders = [JsonFinder(), YamlFinder()]

    def find_module(self, fullname, path=None):
        for finder in self.finders:
            loader = finder.find_module(fullname, path)
            if loader is not None:
                return loader
        return None



# coding: utf-8

import sys
import unittest
import os
import datetime
from pprint import pprint

sys.path.append('../')
import ezcf


class TestProto(unittest.TestCase):

    def test_import(self):
        import sample_json
        self.assertEqual(sample_json.hello, "world")
        self.assertEqual(sample_json.a_list, [1 ,2, 3])
        self.assertEqual(sample_json.a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        import sample_yaml
        self.assertEqual(sample_yaml.Date,
                         datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(sample_yaml.Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            sample_yaml.Stack,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
            {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(sample_yaml.Time,
                         datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(sample_yaml.User, 'ed')
        self.assertEqual(sample_yaml.warning,
                         u'一个 slightly different error message.')
        import sample_yml
        self.assertEqual(sample_yml.Date,
                         datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(sample_yml.Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            sample_yml.Stack,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(sample_yml.Time,
                         datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(sample_yml.User, 'ed')
        self.assertEqual(sample_yml.warning,
                         'A slightly different error message.')

    def test_from_import(self):
        from sample_json import a_list, a_dict
        self.assertEqual(a_list, [1 ,2, 3])
        self.assertEqual(a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        from sample_yaml import Date, Fatal, Stack, Time, User
        self.assertEqual(Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            Stack,
             [{'code': 'x = MoreObject("345\\n")\n',
               'file': 'TopClass.py',
               'line': 23},
              {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(User, 'ed')

        if sys.version_info[:2] > (2, 6):
            with self.assertRaises(NameError):
                print(hello)
            with self.assertRaises(NameError):
                print(warning)

    def test_import_as(self):
        import sample_json as sj
        self.assertEqual(sj.hello, "world")
        self.assertEqual(sj.a_list, [1 ,2, 3])
        self.assertEqual(sj.a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        import sample_yaml as sy
        self.assertEqual(sy.Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(sy.Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            sy.Stack,
             [{'code': 'x = MoreObject("345\\n")\n',
               'file': 'TopClass.py',
               'line': 23},
              {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(sy.Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(sy.User, 'ed')
        self.assertEqual(sy.warning, u'一个 slightly different error message.')

    def test_from_import_as(self):
        from sample_json import hello as h
        from sample_json import a_list as al
        from sample_json import a_dict as ad
        self.assertEqual(h, "world")
        self.assertEqual(al, [1 ,2, 3])
        self.assertEqual(ad, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        from sample_yaml import Date as d
        from sample_yaml import Fatal as f
        from sample_yaml import Stack as s
        from sample_yaml import Time as t
        from sample_yaml import User as u
        from sample_yaml import warning as w
        self.assertEqual(d, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(f, 'Unknown variable "bar"')
        self.assertEqual(
            s,
             [{'code': 'x = MoreObject("345\\n")\n',
               'file': 'TopClass.py',
               'line': 23},
              {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(t, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(u, 'ed')
        self.assertEqual(w, u'一个 slightly different error message.')

    def test_import_subdir(self):
        import subdir.sample_json
        self.assertEqual(subdir.sample_json.hello, "world")
        self.assertEqual(subdir.sample_json.a_list, [1 ,2, 3])
        self.assertEqual(subdir.sample_json.a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        import subdir.sample_yaml
        self.assertEqual(subdir.sample_yaml.Date,
                         datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(subdir.sample_yaml.Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            subdir.sample_yaml.Stack,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(subdir.sample_yaml.Time,
                         datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(subdir.sample_yaml.User, 'ed')
        self.assertEqual(subdir.sample_yaml.warning,
                         'A slightly different error message.')


    def test_from_import_subdir(self):
        from subdir.sample_json import a_list, a_dict
        self.assertEqual(a_list, [1 ,2, 3])
        self.assertEqual(a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        from subdir.sample_yaml import Date, Fatal, Stack, Time, User
        self.assertEqual(Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            Stack,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(User, 'ed')

        if sys.version_info[:2] > (2, 6):
            with self.assertRaises(NameError):
                print(hello)
            with self.assertRaises(NameError):
                print(warning)

    def test_import_as_subdir(self):
        import subdir.sample_json as sj
        self.assertEqual(sj.hello, "world")
        self.assertEqual(sj.a_list, [1 ,2, 3])
        self.assertEqual(sj.a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        import subdir.sample_yaml as sy
        self.assertEqual(sy.Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(sy.Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            sy.Stack,
             [{'code': 'x = MoreObject("345\\n")\n',
               'file': 'TopClass.py',
               'line': 23},
              {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(sy.Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(sy.User, 'ed')
        self.assertEqual(sy.warning, 'A slightly different error message.')

    def test_from_import_as_subdir(self):
        from subdir.sample_json import hello as h
        from subdir.sample_json import a_list as al
        from subdir.sample_json import a_dict as ad
        self.assertEqual(h, "world")
        self.assertEqual(al, [1 ,2, 3])
        self.assertEqual(ad, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        from subdir.sample_yaml import Date as d
        from subdir.sample_yaml import Fatal as f
        from subdir.sample_yaml import Stack as s
        from subdir.sample_yaml import Time as t
        from subdir.sample_yaml import User as u
        from subdir.sample_yaml import warning as w
        self.assertEqual(d, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(f, 'Unknown variable "bar"')
        self.assertEqual(
            s,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(t, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(u, 'ed')
        self.assertEqual(w, 'A slightly different error message.')

    def test_import_subdir2(self):
        import subdir.subdir.sample_json
        self.assertEqual(subdir.subdir.sample_json.hello, "world")
        self.assertEqual(subdir.subdir.sample_json.a_list, [1 ,2, 3])
        self.assertEqual(subdir.subdir.sample_json.a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        import subdir.subdir.sample_yaml
        self.assertEqual(subdir.subdir.sample_yaml.Date,
                         datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(subdir.subdir.sample_yaml.Fatal,
                         'Unknown variable "bar"')
        self.assertEqual(
            subdir.subdir.sample_yaml.Stack,
             [{'code': 'x = MoreObject("345\\n")\n',
               'file': 'TopClass.py',
               'line': 23},
              {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(subdir.subdir.sample_yaml.Time,
                         datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(subdir.subdir.sample_yaml.User, 'ed')
        self.assertEqual(subdir.subdir.sample_yaml.warning,
                         'A slightly different error message.')

    def test_from_import_subdir2(self):
        from subdir.subdir.sample_json import a_list, a_dict
        self.assertEqual(a_list, [1 ,2, 3])
        self.assertEqual(a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        from subdir.subdir.sample_yaml import Date, Fatal, Stack, Time, User
        self.assertEqual(Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            Stack,
             [{'code': 'x = MoreObject("345\\n")\n',
               'file': 'TopClass.py',
               'line': 23},
              {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(User, 'ed')

        if sys.version_info[:2] > (2, 6):
            with self.assertRaises(NameError):
                print(hello)
            with self.assertRaises(NameError):
                print(warning)

    def test_import_as_subdir2(self):
        import subdir.subdir.sample_json as config
        self.assertEqual(config.hello, "world")
        self.assertEqual(config.a_list, [1 ,2, 3])
        self.assertEqual(config.a_dict, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        import subdir.subdir.sample_yaml as sy
        self.assertEqual(sy.Date, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(sy.Fatal, 'Unknown variable "bar"')
        self.assertEqual(
            sy.Stack,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(sy.Time, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(sy.User, 'ed')
        self.assertEqual(sy.warning, 'A slightly different error message.')

    def test_from_import_as_subdir2(self):
        from subdir.sample_json import hello as h
        from subdir.subdir.sample_json import a_list as al
        from subdir.subdir.sample_json import a_dict as ad
        self.assertEqual(h, "world")
        self.assertEqual(al, [1 ,2, 3])
        self.assertEqual(ad, {
            "key1": 1000,
            "key2": [u"你好", 100]
        })
        from subdir.subdir.sample_yaml import Date as d
        from subdir.subdir.sample_yaml import Fatal as f
        from subdir.subdir.sample_yaml import Stack as s
        from subdir.subdir.sample_yaml import Time as t
        from subdir.subdir.sample_yaml import User as u
        from subdir.subdir.sample_yaml import warning as w
        self.assertEqual(d, datetime.datetime(2001, 11, 23, 20, 3, 17))
        self.assertEqual(f, 'Unknown variable "bar"')
        self.assertEqual(
            s,
            [{'code': 'x = MoreObject("345\\n")\n',
              'file': 'TopClass.py',
              'line': 23},
             {'code': 'foo = bar', 'file': 'MoreClass.py', 'line': 58}])
        self.assertEqual(t, datetime.datetime(2001, 11, 23, 20, 2, 31))
        self.assertEqual(u, 'ed')
        self.assertEqual(w, 'A slightly different error message.')

    def test_invalid_json(self):
        from ezcf._base import InvalidJsonError
        if sys.version_info[:2] > (2, 6):
            with self.assertRaises(InvalidJsonError):
                import invalid_json

    def test_invalid_yaml(self):
        from ezcf._base import InvalidYamlError
        if sys.version_info[:2] > (2, 6):
            with self.assertRaises(InvalidYamlError):
                import invalid_yaml



import os
import sys
import codecs
import yaml

from ._base import BaseFinder, BaseLoader, InvalidYamlError


class YamlFinder(BaseFinder):

    def __init__(self, *args, **kwargs):
        super(YamlFinder, self).__init__(*args, **kwargs)

    def find_module(self, fullname, path=None):

        cfg_file = self.get_cfg_filepath(fullname)

        if os.path.isfile(cfg_file + '.yaml'):
            return YamlLoader(cfg_file + '.yaml')
        elif os.path.isfile(cfg_file + '.yml'):
            return YamlLoader(cfg_file + '.yml')
        else:
            return None


class YamlLoader(BaseLoader):

    def __init__(self, cfg_file, *args, **kwargs):
        self.e = None
        self.err_msg = None
        self.cfg_file = cfg_file
        super(YamlLoader, self).__init__(*args, **kwargs)

    def load_module(self, fullname):
        """
        load_module is always called with the same argument as finder's
        find_module, see "How Import Works"
        """
        mod = super(YamlLoader, self).load_module(fullname)

        with codecs.open(self.cfg_file, 'r', 'utf-8') as f:
            try:
                for doc in yaml.load_all(f, yaml.Loader):
                    if isinstance(doc, dict):
                        mod.__dict__.update(doc)
            except yaml.YAMLError:
                self.e = "YAMLError"
                self.err_msg = sys.exc_info()[1]

        if self.e == "YAMLError":
            err_msg = "\nYaml not valid: "
            err_msg += self.cfg_file + '\n'
            err_msg += str(self.err_msg)
            raise InvalidYamlError(err_msg)

        return mod




import os
import json
import sys
import codecs
from ._base import BaseFinder, BaseLoader, InvalidJsonError


class JsonFinder(BaseFinder):

    def __init__(self, *args, **kwargs):
        self.ext_name = ".json"
        super(JsonFinder, self).__init__(*args, **kwargs)

    def find_module(self, fullname, path=None):

        cfg_file = self.get_cfg_filepath(fullname)

        if os.path.isfile(cfg_file + self.ext_name):
            return JsonLoader(cfg_file + self.ext_name)
        else:
            return None


class JsonLoader(BaseLoader):

    def __init__(self, cfg_file, *args, **kwargs):
        self.e = None
        self.err_msg = None
        self.cfg_file = cfg_file
        super(JsonLoader, self).__init__(*args, **kwargs)

    def load_module(self, fullname):
        """
        load_module is always called with the same argument as finder's
        find_module, see "How Import Works"
        """
        mod = super(JsonLoader, self).load_module(fullname)

        try:
            with codecs.open(self.cfg_file, 'r', 'utf-8') as f:
                mod.__dict__.update(json.load(f))
        except ValueError:
            # if raise here, traceback will contain ValueError
            self.e = "ValueError"
            self.err_msg = sys.exc_info()[1]

        if self.e == "ValueError":
            err_msg = "\nJson not valid: "
            err_msg += self.cfg_file + '\n'
            err_msg += str(self.err_msg)
            raise InvalidJsonError(err_msg)

        return mod


__author__ = "laike9m (laike9m@gmail.com)"
__title__ = 'ezcf'
__version__ = '0.0.1'
# __build__ = None
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2015 laike9m'

import sys
from .api import ConfigFinder

sys.meta_path.append(ConfigFinder())


import os

from setuptools import setup, find_packages

def read():
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join('README.rst'), 'r') as f:
        return f.read()

setup(
    name='ezcf',
    version='0.0.1',
    description='Import JSON/YAML like importing .py files',
    long_description=read(),
    url='http://github.com/laike9m/ezcf/',
    license='MIT',
    author='laike9m',
    author_email='laike9m@gmail.com',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data={'': ['README.md']},
    keywords = ['config', 'import'],
    install_requires=['pyyaml'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)